/* -----------------------------------------------------------------------------
 * Copyright (c) 2014 - 2019 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        2021-7-1
 * $Revision:    V1.0.0
 *
 * Project:      Flash Programming Functions for Puya PY32F403xx Flash
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.0.0
 *    Initial release
 */

#include "FlashOS.h"        // FlashOS Structures
#include "py32f410xB.h"

/******************  FLASH Keys  **********************************************/
#define FLASH_KEY1                            (0x45670123UL)
#define FLASH_KEY2                            (0xCDEF89ABUL)

#define FLASH_OPTKEY1                         (0x08192A3BUL)
#define FLASH_OPTKEY2                         (0x4C5D6E7FUL)

#define HW32_REG(ADDRESS)     ( * ((volatile unsigned           int * )(ADDRESS)))

#define HW16_REG(ADDRESS)     ( * ((volatile unsigned short     int * )(ADDRESS)))

#define HW8_REG(ADDRESS)      ( * ((volatile unsigned          char * )(ADDRESS)))

#define M32(ADDRESS)          HW32_REG(ADDRESS)

#define M16(ADDRESS)          HW16_REG(ADDRESS)

#define M8(ADDRESS)           HW8_REG(ADDRESS)

uint32_t gdw_HSI_FS;

/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

#ifdef FLASH_MEM
int Init(unsigned long adr, unsigned long clk, unsigned long fnc)
{

//  InitRccAndFlashParam();

  FLASH->KEYR = FLASH_KEY1;                             // Unlock Flash
  FLASH->KEYR = FLASH_KEY2;

  FLASH->ACR  = 0x00000000;                             // Zero Wait State, no Prefetch
  FLASH->SR  |= FLASH_SR_EOP_Msk;                       // Reset FLASH_EOP

  if ((FLASH->OPTR1 & FLASH_OPTR1_IWDG_SW_Msk) == 0x00) // Test if IWDG is running (IWDG in HW mode)
  {
    // Set IWDG time out to ~32.768 second
    IWDG->KR  = 0x5555;                                 // Enable write access to IWDG_PR and IWDG_RLR
    IWDG->PR  = 0x06;                                   // Set prescaler to 256
    IWDG->RLR = 0xFFF;                                  // Set reload value to 4095
  }

  return (0);
}
#endif

#ifdef FLASH_OPT
int Init(unsigned long adr, unsigned long clk, unsigned long fnc)
{
  FLASH->KEYR = FLASH_KEY1;                             // Unlock Flash
  FLASH->KEYR = FLASH_KEY2;

  FLASH->OPTKEYR = FLASH_OPTKEY1;                       // Unlock Option Bytes
  FLASH->OPTKEYR = FLASH_OPTKEY2;

  FLASH->ACR  = 0x00000000;                             // Zero Wait State, no Prefetch
  FLASH->SR  |= FLASH_SR_EOP_Msk;                              // Reset FLASH_EOP

  if ((FLASH->OPTR1 & FLASH_OPTR1_IWDG_SW_Msk) == 0)            // Test if IWDG is running (IWDG in HW mode)
  {
    // Set IWDG time out to ~32.768 second
    IWDG->KR  = 0x5555;                                 // Enable write access to IWDG_PR and IWDG_RLR
    IWDG->PR  = 0x06;                                   // Set prescaler to 256
    IWDG->RLR = 0xFFF;                                  // Set reload value to 4095
  }

  return (0);
}
#endif


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

#ifdef FLASH_MEM
int UnInit(unsigned long fnc)
{

//  UnInitRccAndFlashParam();

  FLASH->CR    |=  FLASH_CR_LOCK_Msk;                          // Lock Flash

  return (0);
}
#endif

#ifdef FLASH_OPT
int UnInit(unsigned long fnc)
{
  FLASH->CR    |=  FLASH_CR_LOCK_Msk;                          // Lock Flash
  FLASH->CR    |=  FLASH_CR_OPTLOCK_Msk;                       // Lock Option Bytes

  return (0);
}
#endif


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

#ifdef FLASH_MEM
int EraseSector(unsigned long adr)
{
  FLASH->SR  |= FLASH_SR_EOP_Msk;                              // Reset FLASH_EOP

  FLASH->CR  |=  FLASH_CR_SER_Msk;                             // Sector Erase Enabled
  FLASH->CR  |=  FLASH_CR_EOPIE_Msk;
  M32(adr) = 0xFF;                                         // Sector Address
  __asm("DSB");

  while (FLASH->SR & (FLASH_SR_BSY0_Msk|FLASH_SR_BSY1_Msk))
  {
    IWDG->KR = 0xAAAA;                                     // Reload IWDG
  }

  FLASH->CR  &= ~FLASH_CR_SER_Msk;                             // Sector Erase Disabled
  FLASH->CR  &= ~FLASH_CR_EOPIE_Msk;                           // Reset FLASH_EOPIE

  if (FLASH_SR_EOP_Msk != (FLASH->SR & FLASH_SR_EOP_Msk))          // Check for FLASH_SR_EOP
  {
    FLASH->SR |= FLASH_SR_EOP_Msk;
    return (1);                                            // Failed
  }

  return (0);                                              // Done
}
#endif

#ifdef FLASH_OTP
int EraseSector(unsigned long adr)
{
  FLASH->SR  |= FLASH_SR_EOP_Msk;                              // Reset FLASH_EOP

  FLASH->CR  |=  FLASH_CR_PER_Msk;                             // Sector Erase Enabled
  FLASH->CR  |=  FLASH_CR_EOPIE_Msk;
  M32(adr) = 0xFF;                                         // Sector Address
  __asm("DSB");

  while (FLASH->SR & (FLASH_SR_BSY0_Msk|FLASH_SR_BSY1_Msk))
  {
    IWDG->KR = 0xAAAA;                                     // Reload IWDG
  }

  FLASH->CR  &= ~FLASH_CR_PER_Msk;                             // Sector Erase Disabled
  FLASH->CR  &= ~FLASH_CR_EOPIE_Msk;                           // Reset FLASH_EOPIE

  if (FLASH_SR_EOP_Msk != (FLASH->SR & FLASH_SR_EOP_Msk))          // Check for FLASH_SR_EOP
  {
    FLASH->SR |= FLASH_SR_EOP_Msk;
    return (1);                                            // Failed
  }

  return (0);
}
#endif

#ifdef FLASH_OPT
int EraseSector(unsigned long adr)
{
  /* erase sector is not needed for
     - Flash Option bytes
  */
  return (0);
}
#endif


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

#if defined (FLASH_MEM) || defined (FLASH_OTP)
int ProgramPage(unsigned long adr, unsigned long sz, unsigned char *buf)
{
  FLASH->SR  |= FLASH_SR_EOP_Msk;                           // Reset FLASH_EOP
  FLASH->CR  |=  FLASH_CR_PG_Msk;                         // Programming Enabled
  FLASH->CR  |=  FLASH_CR_EOPIE_Msk;

  for (uint32_t i = 0; i < sz / 4; i++)
  {

    M32(adr + i * 4) = M32(buf + i * 4);       // Program the first word of the Double Word
    if (i == (sz / 4 - 2))
    {
      FLASH->CR  |= FLASH_CR_PGSTRT_Msk;
    }
  }
  __asm("DSB");

  while (FLASH->SR & (FLASH_SR_BSY0_Msk|FLASH_SR_BSY1_Msk))
  {
    IWDG->KR = 0xAAAA;                                     // Reload IWDG
  }

  FLASH->CR  &= ~FLASH_CR_PG_Msk;                         // Programming Disabled
  FLASH->CR  &= ~FLASH_CR_EOPIE_Msk;                      // Reset FLASH_EOPIE

  //为了能够Program SRAM，屏蔽下面的EOP检查
//    if (FLASH_SR_EOP != (FLASH->SR & FLASH_SR_EOP)) {   // Check for FLASH_SR_EOP
//      FLASH->SR |= FLASH_SR_EOP;
//      return (1);                                       // Failed
//    }

  return (0);                                           // Done
}
#endif //FLASH_MEM

#ifdef FLASH_OPT
int ProgramPage(unsigned long adr, unsigned long sz, unsigned char *buf)
{
  uint32_t optr1,optr2,wrpr0,wrpr1,pcrop0sr,pcrop0er,pcrop1sr,pcrop1er;

  if(adr!=0x1FFF1100)
    return 1;

  optr1 = M16(buf);
  optr2 = M16(buf+0x4);
  wrpr0= M16(buf+0x8);
  wrpr1= M16(buf+0xC);
  pcrop0sr= M16(buf+0x10);
  pcrop0er= M16(buf+0x14);
  pcrop1sr= M16(buf+0x18);
  pcrop1er= M16(buf+0x1C);

  FLASH->SR |= FLASH_SR_EOP_Msk;                            // Reset FLASH_EOP

  FLASH->OPTR1 = optr1;
  FLASH->OPTR2 = optr2;
  FLASH->BANK0_WRPR = wrpr0;
  FLASH->BANK1_WRPR = wrpr1;
  FLASH->PCROP0SR = pcrop0sr;
  FLASH->PCROP0ER = pcrop0er;
  FLASH->PCROP1SR = pcrop1sr;
  FLASH->PCROP1ER = pcrop1er;

  FLASH->CR |= FLASH_CR_OPTSTRT_Msk;
  FLASH->CR  |= FLASH_CR_EOPIE_Msk;

  __asm("DSB");

  while (FLASH->SR & (FLASH_SR_BSY0_Msk|FLASH_SR_BSY1_Msk))
  {
    IWDG->KR = 0xAAAA;                                     // Reload IWDG
  }

  FLASH->CR  &= ~FLASH_CR_OPTSTRT_Msk;                      // Programming Disabled
  FLASH->CR  &= ~FLASH_CR_EOPIE_Msk;                        // Reset FLASH_EOPIE

  return (0);                                           // Done
}
#endif //  FLASH_OPT

